part of dashboard;

class DashboardItem {
  final String id;
  final String label;
  final Widget icon;
  final Widget? selectedIcon;
  final Widget Function()? _builder;
  final List<NavigatorItem> _navigatorPages;
  final bool _reportsRouteUpdateToEngine;
  final List<NavigatorObserver> _observers;
  final GlobalKey<NavigatorState>? _navigatorKey;
  final void Function(String? value)? search;
  final AppBarOptions? appBarOptions;
  final List<DashboardItem> subItems;
  final bool Function(Route<dynamic>, dynamic)? _onPopPage;

  /// {@macro flutter.material.appBar.actions}
  ///
  /// This property is used to configure an [AppBar].
  final List<Widget> actions;

  final bool overrideActions;
  bool get hasBuilder => _builder != null || _navigatorPages.isNotEmpty;
  DashboardItem({
    required this.label,
    required this.id,
    required Widget Function() builder,
    required this.icon,
    this.appBarOptions,
    this.selectedIcon,
    this.search,
    this.overrideActions = false,
    this.actions = const [],
  })  : subItems = [],
        _builder = builder,
        _navigatorKey = null,
        _onPopPage = null,
        _navigatorPages = [],
        _observers = [],
        _reportsRouteUpdateToEngine = false;

  DashboardItem.withNavigator({
    required this.label,
    required this.id,
    required List<NavigatorItem> pages,
    required this.icon,
    GlobalKey<NavigatorState>? navigatorKey,
    this.appBarOptions,
    bool reportsRouteUpdateToEngine = false,
    this.selectedIcon,
    this.search,
    this.overrideActions = false,
    bool Function(Route<dynamic>, dynamic)? onPopPage,
    this.actions = const [],
    List<NavigatorObserver> observers = const [],
  })  : subItems = [],
        assert(pages.isNotEmpty),
        _observers = observers,
        _onPopPage = onPopPage,
        _builder = null,
        _navigatorKey = navigatorKey,
        _navigatorPages = pages,
        _reportsRouteUpdateToEngine = reportsRouteUpdateToEngine;
  DashboardItem.items({
    required this.label,
    required this.icon,
    required this.subItems,
  })  : selectedIcon = null,
        appBarOptions = null,
        id = '',
        search = null,
        _onPopPage = null,
        overrideActions = false,
        _navigatorKey = null,
        actions = const [],
        _observers = const [],
        _builder = null,
        _reportsRouteUpdateToEngine = false,
        _navigatorPages = [];
}

class PageInfo extends StatefulWidget {
  final DashboardItem? _item;
  final List<TextSpan> titleSpans;
  final Widget? _child;
  const PageInfo({
    Key? key,
    required DashboardItem item,
    required this.titleSpans,
  })  : _child = null,
        _item = item,
        super(key: key);
  const PageInfo.child({
    Key? key,
    required Widget child,
    required this.titleSpans,
  })  : _child = child,
        _item = null,
        super(key: key);
  @override
  State<PageInfo> createState() => _PageInfoState();

  static PageInfo? of(BuildContext context) =>
      context.findAncestorStateOfType<_PageInfoState>()?.widget;
}

class _PageInfoState extends State<PageInfo> {
  @override
  Widget build(BuildContext context) {
    var item = widget._item;
    if (item == null) {
      return widget._child ?? const SizedBox();
    }
    if (item._builder != null) {
      return item._builder!();
    }
    List<Page> pages = [];
    for (var i = 0; i < item._navigatorPages.length; i++) {
      var page = item._navigatorPages[i];
      pages.add(
        MaterialPage(
          key: ValueKey(page.name.isEmpty ? '$i' : page.name),
          child: page.child,
        ),
      );
    }
    return Navigator(
      onPopPage: item._onPopPage ?? (_, __) => false,
      initialRoute: item._navigatorPages.first.name,
      key: item._navigatorKey,
      reportsRouteUpdateToEngine: item._reportsRouteUpdateToEngine,
      observers: item._observers,
      pages: pages,
    );
  }
}

class NavigatorItem {
  final String name;
  final Widget child;
  NavigatorItem({
    required this.name,
    required this.child,
  });
}
